Rem
bbdoc: Advanced Body object
EndRem
Type TAdvBody Extends TBody
	Global array:TAdvBody[]
	Global aNext:Int=0
	Global stack:Int[]
	Global sNext:Int=0
	
	Field mIndex:Int=0
	Field mArray:TMass[]
	Field cIndex:Int=0
	Field cArray:TConstraint[]
	Field iIndex:Int=0
	Field iArray:TImage[]
	Field rIndex:Int=0
	Field rArray:TRig[]
	
	Field aIndex:Int=-1
	
	Rem
	bbdoc: Loads a previously saved advanced body object.
	returns: Loaded TAdvBody.
	about: The Load function loads a TAdvBody that has previously been saved using the #Save method.
	The @active parameter determines if the TAdvBody will be #Activated immediately after loading.
	The loaded TAdvBody will be identical to the original,
	however collision groups need a little further explination.
	If a mass was created using an absolute value for it's @colGroup, i.e. 0 or above,
	then the corrisponding loaded mass will have the exact same colGroup.
	If however the original mass was created using -1 as it's colGroup the loaded mass will be given the next unique colGroup.
	Masses within the same TAdvBody that share the same colGroup will keep those associations after loading.
	End Rem
	Function Load:TAdvBody( url:Object, active:Int=False )
		'setup file ready for reading
		Local file:TStream=ReadStream(url)
		If Not file Then Return Null
		file=LittleEndianStream(file)
		
		'read header
		Local header:String=ReadString(file,4)
		Local version:Int
		If header="BODY"
			version=103
		ElseIf Left(header,1)="B"
			version=Int(Right(header,3))
		Else
			CloseStream file
			Return Null
		EndIf
		
		'Create advanced body
		Local ab:TAdvBody=New TAdvBody
		
		ab.SetName ReadLine(file)
		
		'load masses
		ab.mIndex=ReadInt(file)
		ab.mArray=New TMass[ab.mIndex]
		
		Local colsrc:Int[ab.mIndex]
		Local coldes:Int[ab.mIndex]
		Local ncgs:Int=0
		
		Local index:Int=ReadInt(file)
		While index<>-1
			
			Local posx:Float=ReadFloat(file)
			Local posy:Float=ReadFloat(file)
			Local mass:Float=ReadFloat(file)
			Local radi:Float=ReadFloat(file)
			Local fric:Float=ReadFloat(file)
			Local cg:Int=ReadInt(file)
			
			ab.mArray[index]=TMass.Create(posx,posy,mass,radi,fric,cg)
			
			If cg<0
				Local found:Int=False
				For Local count:Int=0 To ncgs-1
					If colsrc[count]=cg
						found=True
						ab.mArray[index].SetColGroup coldes[count]
					EndIf
				Next
				If Not found
					ab.mArray[index].SetColGroup -1
					colsrc[ncgs]=cg
					coldes[ncgs]=ab.mArray[index].GetColGroup()
					ncgs:+1
				EndIf
			EndIf
			
			ab.mArray[index].SetName ReadLine(File)
			If ReadByte(file) Then ab.mArray[index].Freeze
			
			Local velx:Float=ReadFloat(file)
			Local vely:Float=ReadFloat(file)
			ab.mArray[index].SetVelocity velx,vely
			ab.mArray[index].abID=ab.aIndex
			ab.mArray[index].index=index
			
			index=ReadInt(file)
		Wend
		
		'load constraints
		ab.cIndex=ReadInt(file)
		ab.cArray=New TConstraint[ab.cIndex]
		
		index=ReadInt(file)
		While index<>-1
			
			Local m1:Int=ReadInt(file)
			Local m2:Int=ReadInt(file)
			Local rl:Float=ReadFloat(file)
			Local s:Float=ReadFloat(file)
			
			ab.cArray[index]=TConstraint.Create( ab.mArray[m1],ab.mArray[m2],rl,s )
			ab.cArray[index].SetName ReadLine(file)
			ab.cArray[index].abID=ab.aIndex
			ab.cArray[index].index=index
			
			index=ReadInt(file)
		Wend
		
		'load images
		Local packed:Int=False
		If version>110
			packed=ReadByte(file)
		EndIf
		
		ab.iIndex=ReadInt(file)
		ab.iArray=New TImage[ab.iIndex]
		
		index=ReadInt(file)
		While index<>-1
			
			Local mr:Int=ReadByte(file)
			Local mg:Int=ReadByte(file)
			Local mb:Int=ReadByte(file)
			SetMaskColor mr,mg,mb
			
			Local flags:Int=ReadByte(file)
			Local hx:Float=ReadFloat(file)
			Local hy:Float=ReadFloat(file)
			If packed
				ab.iArray[index]=LoadImage(file,flags)
			Else
				ab.iArray[index]=LoadImage(StripExt(RealPath(String(url)))+"_img"+index+".png",flags)
			EndIf
			SetImageHandle ab.iArray[index],hx,hy
			
			index=ReadInt(file)
		Wend
		
		'load rigs
		ab.rIndex=ReadInt(file)
		ab.rArray=New TRig[ab.rIndex]
		
		index=ReadInt(file)
		While index<>-1
			
			Local img:TImage=ab.iArray[ReadInt(file)]
			Local mass:TMass=ab.mArray[ReadInt(file)]
			Local con:TConstraint=Null
			Local ind:Int=ReadInt(file)
			If ind>-1 Then con=ab.cArray[ind]
			Local angle:Float=ReadFloat(file)
			
			ab.rArray[index]=TRig.Create(img,mass,con,angle)
			
			If version>109
				Local sclX:Float=ReadFloat(file)
				Local sclY:Float=ReadFloat(file)
				Local red:Int=ReadByte(file)
				Local green:Int=ReadByte(file)
				Local blue:Int=ReadByte(file)
				Local alpha:Float=ReadFloat(file)
				Local blend:Int=ReadByte(file)
				
				ab.rArray[index].SetScale sclX,sclY
				ab.rArray[index].SetColor red,green,blue
				ab.rArray[index].SetAlpha alpha
				ab.rArray[index].SetBlend blend
			EndIf
			
			ab.rArray[index].SetName ReadLine(file)
			ab.rArray[index].abID=ab.aIndex
			ab.rArray[index].index=index
			
			index=ReadInt(file)
		Wend
		
		
		CloseStream file
		
		If active Then ab.Activate
		
		Return ab
		
	End Function
	
	Rem
	bbdoc: Counts all TAdvBodies.
	returns: The number of TAdvBodies.
	about: CountAll will include TAdvBodies that are not active.
	End Rem
	Function CountAll:Int()
		Return aNext-sNext
	End Function
	
	Rem
	bbdoc: Maps one function to all TAdvBodies.
	about: ApplyFunc is the function that will be run for every active TAdvBody in the simulation.
	End Rem
	Function ApplyToAll( ApplyFunc( advBody:TAdvBody ) )
		For Local count:Int=0 Until aNext
			If array[count]
				ApplyFunc array[count]
			EndIf 
		Next
	End Function
	
	Rem
	bbdoc: Saves the advanced body object.
	about:
	Saves a #TAdvBody to the location specified by @url.
	If no extension is included in @url then the extension ".plb" will be added.
	
	By default Save packs the any images used by the body into the .plb file.
	If @packImages is set to %false then images are saved alongside the save file with the suffix "_img00.png",
	where 00 is the index of the image within the #TAdvBody.
	End Rem
	Method Save( url:Object, backup:Int=True, packImages:Int=True )
		Local file:TStream=TStream(url)
		Local str_url:String=String(url)
		If file
			file=WriteStream(file)
			packImages=True
		ElseIf str_url
			str_url=RealPath(str_url)
			Select FileType(str_url)
			Case 2
				Return
			Case 1
				If backup Then CopyFile str_url,str_url+".bak"
				DeleteFile str_url
			End Select
			If str_url=StripExt(str_url) Then str_url=str_url+".plb"
			file=WriteStream(str_url)
		Else
			Return
		EndIf
		
		file=LittleEndianStream(file)
		
		WriteString file,"B112"
		WriteLine file,GetName()
		
		WriteInt file,mIndex
		For Local mass:TMass=EachIn mArray
			WriteInt file,mass.index
			WriteFloat file,mass.GetX()
			WriteFloat file,mass.GetY()
			WriteFloat file,mass.GetMass()
			WriteFloat file,mass.GetRadius()
			WriteFloat file,mass.friction
			WriteInt file,mass.GetColGroup()
			WriteLine file,mass.GetName()
			WriteByte file,mass.IsFrozen()
			WriteFloat file,mass.GetVX()
			WriteFloat file,mass.GetVY()
		Next
		
		WriteInt file,-1
		
		WriteInt file,cIndex
		For Local con:TConstraint=EachIn cArray
			WriteInt file,con.index
			WriteInt file,con.GetMass1().index
			WriteInt file,con.GetMass2().index
			WriteFloat file,con.GetRestLength()
			WriteFloat file,con.GetStiffness()
			WriteLine file,con.GetName()
		Next
		
		WriteInt file,-1
		
		WriteByte file,packImages
		
		WriteInt file,iIndex
		For Local index:Int=0 To iIndex-1
			If iArray[index]
				WriteInt file,index
				WriteByte file,iArray[index].mask_r
				WriteByte file,iArray[index].mask_g
				WriteByte file,iArray[index].mask_b
				WriteByte file,iArray[index].flags
				WriteFloat file,iArray[index].handle_x
				WriteFloat file,iArray[index].handle_y
				If packImages
					SavePixmapPNG iArray[index].pixmaps[0],file,9
				Else
					SavePixmapPNG iArray[index].pixmaps[0],StripExt(str_url)+"_img"+index+".png"
				EndIf
			EndIf
		Next
		
		WriteInt file,-1
		
		WriteInt file,rIndex
		For Local index:Int=0 To rIndex-1
			If rArray[index]
				WriteInt file,index
				Local count:Int=0
				While iArray[count]<>rArray[index].image
					count:+1
				Wend
				WriteInt file,count
				WriteInt file,rArray[index].mass.index
				If rArray[index].con
					WriteInt file,rArray[index].con.index
				Else
					WriteInt file,-1
				EndIf
				WriteFloat file,rArray[index].angle
				
				WriteFloat file,rArray[index].scaleX
				WriteFloat file,rArray[index].scaleY
				Local r:Int,g:Int,b:Int
				rArray[index].GetColor r,g,b
				WriteByte file,r
				WriteByte file,g
				WriteByte file,b
				WriteFloat file,rArray[index].alpha
				WriteByte file,rArray[index].blend
				
				WriteLine file,rArray[index].GetName()
			EndIf
		Next
		
		WriteInt file,-1
		
		CloseStream file
		
	End Method
	
	Rem
	bbdoc: Copies the advanced body object.
	returns: The copied TAdvBody.
	about: Creates an exact copy of the TAdvBody.
	If active is set to false the TAdvBody will not be active immediately and needs to be activated before use.
	TImages included in the source TAdvBody are not copied, but are instead shared by the original and copy.
	If a mass from the source TAdvBody was created using an absolute value for it's @colGroup, i.e. 0 or above,
	then the corrisponding copied mass will have the exact same colGroup.
	If however the original mass was created using -1 as it's colGroup then the copied mass will be given the next unique colGroup.
	Masses within the same TAdvBody that share the same colGroup will keep those associations after copying.
	End Rem
	Method Copy:TAdvBody( active:Int=False )
		Local ab:TAdvBody=New TAdvBody
		
		ab.SetName GetName()
		
		ab.mIndex=mIndex
		ab.mArray=New TMass[mIndex]
		ab.cIndex=cIndex
		ab.cArray=New TConstraint[cIndex]
		ab.iIndex=iIndex
		ab.iArray=New TImage[iIndex]
		ab.rIndex=rIndex
		ab.rArray=New TRig[rIndex]
		
		Local colsrc:Int[ab.mIndex]
		Local coldes:Int[ab.mIndex]
		Local ncgs:Int=0
		
		For Local index:Int=0 To mIndex-1
			If mArray[index]
				ab.mArray[index]=mArray[index].Copy()
				ab.mArray[index].abID=ab.aIndex
				Local cg:Int=ab.mArray[index].GetColGroup()
				If cg<0
					Local found:Int=False
					For Local count:Int=0 To ncgs-1
						If colsrc[count]=cg
							found=True
							ab.mArray[index].SetColGroup coldes[count]
						EndIf
					Next
					If Not found
						ab.mArray[index].SetColGroup -1
						colsrc[ncgs]=cg
						coldes[ncgs]=ab.mArray[index].GetColGroup()
						ncgs:+1
					EndIf
				EndIf
			EndIf
		Next
		For Local index:Int=0 To cIndex-1
			If cArray[index]
				Local m1:TMass=ab.mArray[ConGetMass1(index)]
				Local m2:TMass=ab.mArray[ConGetMass2(index)]
				ab.cArray[index]=TConstraint.Create(m1,m2,ConGetRestLength(index),ConGetStiffness(index))
				ab.cArray[index].index=index
				ab.cArray[index].abID=ab.aIndex
				ab.ConSetName index,ConGetName(index)
			EndIf
		Next
		For Local index:Int=0 To iIndex-1
			ab.iArray[index]=iArray[index]
		Next
		For Local index:Int=0 To rIndex-1
			If rArray[index]
				Local img:TImage=rArray[index].image
				Local mass:TMass=ab.mArray[rArray[index].mass.index]
				Local con:TConstraint=Null
				If rArray[index].con Then con=ab.cArray[rArray[index].con.index]
				ab.rArray[index]=TRig.Create(img,mass,con,rArray[index].angle)
				ab.RigSetName index,RigGetName(index)
				Local sx:Float,sy:Float
				RigGetScale index,sx,sy
				ab.RigSetScale index,sx,sy
				Local red:Int,green:Int,blue:Int
				RigGetColor index,red,green,blue
				ab.RigSetColor index,red,green,blue
				ab.RigSetAlpha index,RigGetAlpha(index)
				ab.RigSetBlend index,RigGetBlend(index)
				ab.rArray[index].abID=ab.aIndex
				ab.rArray[index].index=index
			EndIf
		Next
		
		If active Then ab.Activate
		
		Return ab
	End Method
	
	Rem
	bbdoc: Removes blank indices from the TAdvBody.
	about:
	CleanIndex condenses the mass, con, rig and image indices of the #TAdvBody.
	This means that all gaps(blank index places) are filled by higher indexed components.
	
	@Note: After using CleanIndex the index values of each component may have changed.
	End Rem
	Method CleanIndex()
		MassCleanIndex
		ConCleanIndex
		RigCleanIndex
		ImageCleanIndex
	End Method
	
	Rem
	bbdoc: Activates the TAdvBody
	about:
	When created a #TAdvBody is not active within the simulation.
	Using Activate is the equivilant of assigning all the constraints within the #TAdvBody.
	
	See Also: #DeActivate
	End Rem
	Method Activate()
		For Local index:Int=0 To cIndex-1
			ConAssign index
		Next
	End Method
	
	Rem
	bbdoc: Deactivates the TAdvBody
	about: Stops the #TAdvBody from being active in the simulation.
	End Rem
	Method DeActivate()
		For Local index:Int=0 To cIndex-1
			ConFree index
		Next
	End Method
	
	Rem
	bbdoc: Frees the TAdvBody
	about: Use free when the TAdvBody is no longer needed to free it from memory.
	End Rem
	Method Free()
		If aIndex=-1 Then Return
		For Local count:Int=0 Until rIndex
			RigRemove count
		Next
		For Local count:Int=0 Until iIndex
			ImageRemove count
		Next
		For Local count:Int=0 Until cIndex
			ConRemove count
		Next
		For Local count:Int=0 Until mIndex
			MassRemove count
		Next
		DeActivate
		stack[sNext]=aIndex
		sNext:+1
		array[aIndex]=Null
		aIndex=-1
		If aNext=sNext
			aNext=0
			sNext=0
		EndIf
	End Method
	
	Rem
	bbdoc: Draws the TAdvBody
	about: the Draw method draws all the rigs associated with the TAdvBody
	End Rem
	Method Draw()
		For Local index:Int=0 To rIndex-1
			If rArray[index] Then RigDraw index
		Next
	End Method
	
	Rem
	bbdoc: Moves the TAdvBody
	about: The Move method moves all the masses associated with the TAdvBody
	End Rem
	Method Move( x:Float, y:Float )
		For Local index:Int=0 To mIndex-1
			MassMove index,x,y
		Next
	End Method
	
	Rem
	bbdoc: Translates the TAdvBody
	about: The Translate method translates all the masses associated with the TAdvBody.
	End Rem
	Method Translate( x:Float, y:Float )
		For Local index:Int=0 To mIndex-1
			MassTranslate index,x,y
		Next
	End Method
	
	Rem
	bbdoc: Position the TAdvBody
	about: The Position method positions the TAdvBody at position x,y.
	The bodies handle is the mass with the lowest index value, usually 0.
	End Rem
	Method Position( x:Float, y:Float)
		Local index:Int=0
		While index<mIndex And mArray[index]=Null
			index:+1
		Wend
		If index<mIndex
			Local ox:Float=mArray[index].GetX()
			Local oy:Float=mArray[index].GetY()
			For index=0 To mIndex-1
				If mArray[index] Then MassPosition index,x+mArray[index].GetX()-ox,y+mArray[index].GetY()-oy
			Next
		EndIf
	End Method
	
	Rem
	bbdoc: Gets the position of the TAdvBody.
	about: GetPosition assigns the position of the body to the values x,y.
	The position is in fact the position of the mass with the lowest index, usually 0.
	End Rem
	Method GetPosition( x:Float Var, y:Float Var )
		Local index:Int=0
		While index<mIndex And mArray[index]=Null
			index:+1
		Wend
		If index<mIndex Then MassGetPosition index,x,y
	End Method
	
	Rem
	bbdoc: Gets the X coordinate of the TAdvBody position.
	returns: the X coordinate of the TAdvBody.
	about: See also: #GetPosition.
	End Rem
	Method GetX:Float()
		Local index:Int=0
		While index<mIndex And mArray[index]=Null
			index:+1
		Wend
		If index<mIndex Then Return MassGetX(index)
	End Method
	
	Rem
	bbdoc: Gets the Y coordinate of the TAdvBody position.
	returns: the Y coordinate of the TAdvBody.
	about: See also: #GetPosition.
	End Rem
	Method GetY:Float()
		Local index:Int=0
		While index<mIndex And mArray[index]=Null
			index:+1
		Wend
		If index<mIndex Then Return MassGetY(index)
	End Method
	
	Rem
	bbdoc: Rotates TAdvBody
	about: The Rotate method rotates the TAdvBoby to an absolute @angle.
	Rotate also set the TAdvBody velocity to 0.
	Think of Rotate as using #MassPosition on all the masses to rotate them to a set @angle.
	End Rem
	Method Rotate( angle:Float )
		Local ox:Float,oy:Float
		GetPosition ox,oy
		Local oang:Float=GetRotation()
		For Local mass:TMass=EachIn mArray
			Local mx:Float=mass.GetX()-ox
			Local my:Float=mass.GetY()-oy
			Local mag:Float=Sqr( mx*mx + my*my )
			Local mang:Float=ATan2(my,mx)-oang
			mass.Position ox+Cos(mang+angle)*mag,oy+Sin(mang+angle)*mag
		Next
	End Method
	
	Rem
	bbdoc: Gets the angle of the TAdvBody
	returns: The angle of the constraint with the lowest index.
	End Rem
	Method GetRotation:Float()
		Local index:Int=0
		While index<cIndex And cArray[index]=Null
			index:+1
		Wend
		If index<cIndex Then Return ConGetAngle(index)
	End Method
	
	Rem
	bbdoc: Spins the TAdvBody
	about: The #Spin method spins the TAdvBody by @angle degrees.
	This will also add velocity to the body meaning it will continue to spin.
	Think of Spin as performing #MassMove on all the masses to spin the body @angle degrees.
	End Rem
	Method Spin( angle:Float )
		Local ox:Float,oy:Float
		GetPosition ox,oy
		For Local mass:TMass=EachIn mArray
			Local mx:Float=mass.GetX()-ox
			Local my:Float=mass.GetY()-oy
			Local mag:Float=Sqr( mx*mx + my*my )
			Local mang:Float=ATan2(my,mx)+angle
			mass.Move Cos(mang)*mag-mx,Sin(mang)*mag-my
		Next
	End Method
	
	Rem
	bbdoc: Turns the TAdvBody
	about: The Turn method turns the TAdvBody @angle degrees.
	Turn leave the bodies current velocity unaffected.
	Think of Turn as performing #MassTranslate on all the masses to turn the body @angle degrees.
	End Rem
	Method Turn( angle:Float )
		Local ox:Float,oy:Float
		GetPosition ox,oy
		For Local mass:TMass=EachIn mArray
			Local mx:Float=mass.GetX()-ox
			Local my:Float=mass.GetY()-oy
			Local mag:Float=Sqr( mx*mx + my*my )
			Local mang:Float=ATan2(my,mx)+angle
			mass.Translate Cos(mang)*mag-mx,Sin(mang)*mag-my
		Next
	End Method
	
	Rem
	bbdoc: Freezes all the bodies masses
	End Rem
	Method Freeze()
		For Local count:Int=0 Until mIndex
			If mArray[count] Then mArray[count].Freeze
		Next
	End Method
	
	Rem
	bbdoc: Thaws all the bodies masses
	End Rem
	Method Thaw()
		For Local count:Int=0 Until mIndex
			If mArray[count] Then mArray[count].Thaw
		Next
	End Method
	
	Rem
	bbdoc: Retrieves the collisions between #TAdvBody's.
	returns: A list of Collision Events.
	about:
	Checks for collisions between the #TMass objects of two Advanced Bodys and returns a list of all the #TColEvent objects.
	
	See Also: #CollideZone, #CollideBounds and #TColEvent
	End Rem
	Method CollideAdvBody:TList( advbody:TAdvBody )
		Local list:TList=New TList
		For Local src:TMass=EachIn mArray
			For Local tar:TMass=EachIn advbody.mArray
				Local col:TColEvent=TColEvent.Seek(src,tar)
				If col Then list.AddLast col
			Next
		Next
		If list.IsEmpty() Then Return Null Else Return list
	End Method
	
	Rem
	bbdoc: Retreives all the collisions between the #TAdvBody and a #TZone.
	returns: A list of Collision Events.
	about:
	Checks for collisions between the #TMass objects of the #TAdvBody and a #TZone, and returns a list of all the TColEvent objects.
	
	See Also: #CollideAdvBody, #CollideBounds and #TColEvent
	End Rem
	Method CollideZone:TList( zone:TZone )
		Local list:TList=New TList
		For Local mass:TMass=EachIn mArray
			Local col:TColEvent=TColEvent.Seek(mass,zone)
			If col Then list.AddLast col
		Next
		If list.IsEmpty() Then Return Null Else Return list
	End Method
	
	Rem
	bbdoc: Retreives all the collisions between the #TAdvBody and the simulations bounds.
	returns: A list of Collision Events.
	about:
	Checks for collisions between the #TMass objects of the #TAdvBody and the simulation bounds,
	and returns a list of all the #TColEvent objects.
	
	See Also: #CollideAdvBody, #CollideZone and #TColEvent
	End Rem
	Method CollideBounds:TList()
		Local list:TList=New TList
		For Local mass:TMass=EachIn mArray
			Local col:TColEvent=TColEvent.Seek(mass)
			If col Then list.AddLast col
		Next
		If list.IsEmpty() Then Return Null Else Return list
	End Method
	
	Rem
	bbdoc: Creates and adds a mass to the TAdvBody
	returns: The index of the created mass
	about: Creates a mass associated with the TAdvBody and returns it's index.
	See TMass.Create for more information.
	End Rem
	Method MassAdd:Int( x:Float, y:Float, mass:Float, radius:Float, friction:Float, colGroup:Int=-1 )
		mIndex:+1
		Local m:TMass=TMass.Create( x, y, mass, radius, friction, colGroup )
		If mArray.Length<mIndex Then mArray=mArray[..mIndex*2]
		mArray[mIndex-1]=m
		m.abID=aIndex
		m.index=mIndex-1
		
		Return mIndex-1
	End Method
	
	Rem
	bbdoc: Removes a mass from the TAdvBody
	about: Removes the mass at index from the TAdvBody.
	Remove will also remove any constraint and rigs that reference it.
	It should be noted that index values are not recycled.
	So removing a mass will leave a gap in the index range.
	End Rem
	Method MassRemove( index:Int )
		If mArray[index]
			For Local rcount:Int=0 To rIndex-1
				If rArray[rcount]
					If rArray[rcount].mass=mArray[index] Then RigRemove rcount
				EndIf
			Next
			For Local ccount:Int=0 To cIndex-1
				If cArray[ccount]
					If cArray[ccount].m[0]=mArray[index] Or cArray[ccount].m[1]=mArray[index]
						ConRemove ccount
					EndIf
				EndIf
			Next
			mArray[index].abID=-1
			mArray[index]=Null
		EndIf
	End Method
	
	Rem
	bbdoc: Get the maximum mass index used by the TAdvBody
	returns: Maximum index
	about: note MassMaxIndex returns the maximum index that has been used by the TAdvBody.
	The actual maximum index may be lower.
	End Rem
	Method MassMaxIndex:Int()
		Return mIndex-1
	End Method
	
	Rem
	bbdoc: Swaps the index positions of two masses.
	End Rem
	Method MassSwapIndex( index1:Int, index2:Int )
		If index1<0 Or index2<0 Or index1>=mIndex Or index2>=mIndex Then Return
		
		Local swap:TMass=mArray[index1]
		mArray[index1]=mArray[index2]
		mArray[index2]=swap
		
		If mArray[index1] Then mArray[index1].index=index1
		If mArray[index2] Then mArray[index2].index=index2
	End Method
	
	Rem
	bbdoc: Condenses the indices of the masses.
	about: MassCleanIndex removes gaps in the index list.
	It should be noted that masses index values will be changed by this method.
	End Rem
	Method MassCleanIndex()
		Local freeIndex:Int=0
		
		For Local count:Int=0 To mIndex-1
			
			If mArray[count]
				MassSwapIndex( freeIndex, count )
				
				freeIndex:+1
				
			EndIf
			
		Next
		
		mIndex=freeIndex
	End Method
	
	Rem
	bbdoc: Sees if mass exists
	returns: True if mass exists, false if otherwise.
	about: Not all indices within the index range are used by A TAdvBody.
	Use MassExists to see if @index is currently in use.
	End Rem
	Method MassExists:Int( index:Int )
		If index>=0 And index<mIndex And mArray[index] Then Return True
		Return False
	End Method
	
	Rem
	bbdoc: Gets a TMass component.
	returns: The TMass at index.
	about: MassGet returns the TMass at the given index.
	End Rem
	Method MassGet:TMass( index:Int )
		If mArray[index] Then Return mArray[index]
	End Method
	
	Method MassSetName( index:Int, name:String )
		If mArray[index] Then mArray[index].SetName name
	End Method
	
	Method MassGetName:String( index:Int )
		If mArray[index] Then Return mArray[index].GetName()
	End Method
	
	Method MassMove( index:Int, x:Float, y:Float )
		If mArray[index] Then mArray[index].Move x,y
	End Method
	
	Method MassTranslate( index:Int, x:Float, y:Float )
		If mArray[index] Then mArray[index].Translate x,y
	End Method
	
	Method MassPosition( index:Int, x:Float, y:Float )
		If mArray[index] Then mArray[index].Position x,y
	End Method
	
	Method MassGetPositionX:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetPositionX()
	End Method
	
	Method MassGetPositionY:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetPositionY()
	End Method
	
	Method MassGetPosition( index:Int, x:Float Var, y:Float Var )
		If mArray[index] Then mArray[index].GetPosition x,y
	End Method
	
	Method MassGetX:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetX()
	End Method
	
	Method MassGetY:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetY()
	End Method
	
	Method MassSetMass( index:Int, m:Float )
		If mArray[index] Then mArray[index].SetMass m
	End Method
	
	Method MassGetMass:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetMass()
	End Method
	
	Method MassSetRadius( index:Int, r:Float )
		If mArray[index] Then mArray[index].SetRadius r
	End Method
	
	Method MassGetRadius:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetRadius()
	End Method
	
	Method MassSetFriction( Index:Int, f:Float )
		If mArray[index] Then mArray[index].SetFriction f
	End Method
	
	Method MassGetFriction:Float( Index:Int )
		If mArray[index] Then Return mArray[index].GetFriction()
	End Method
	
	Method MassFreeze( index:Int )
		If mArray[index] Then mArray[index].Freeze
	End Method
	
	Method MassThaw( index:Int )
		If mArray[index] Then mArray[index].Thaw
	End Method
	
	Method MassIsFrozen:Int( index:Int )
		If mArray[index] Then Return mArray[index].IsFrozen() 
	End Method
	
	Method MassSetVelocity( index:Int, x:Float, y:Float )
		If mArray[index] Then mArray[index].SetVelocity x,y
	End Method
	
	Method MassAddVelocity( index:Int, x:Float, y:Float )
		If mArray[index] Then mArray[index].AddVelocity x,y
	End Method
	
	Method MassGetVelocityX:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetVelocityX()
	End Method
	
	Method MassGetVelocityY:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetVelocityY()
	End Method
	
	Method MassGetVX:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetVX()
	End Method
	
	Method MassGetVY:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetVY()
	End Method
	
	Method MassGetVelocity:Float( index:Int )
		If mArray[index] Then Return mArray[index].GetVelocity()
	End Method
	
	Method MassGetColGroup:Int( index:Int )
		If mArray[index] Then Return mArray[index].GetColGroup()
	End Method
	
	Method MassSetColGroup( index:Int, value:Int=-1 )
		If mArray[index] Then mArray[index].SetColGroup value
	End Method
	
	Method MassIsActive:Int( index:Int )
		If mArray[index] Then Return mArray[index].IsActive()
	End Method
	
	Method MassDraw( index:Int )
		If mArray[index] Then mArray[index].Draw
	End Method
	
	Method MassDrawAll()
		For Local m:TMass=EachIn mArray
			m.Draw
		Next
	End Method
	
	Rem
	bbdoc: A shortcut method to connect all the masses in a TAdvBody
	about: ConAddAll does not make efficient TAdvBodys and should be used sparingly.
	ConAddAll also does not take into account already created constraints and will result in these being duplicated.
	End Rem
	Method ConAddAll( stiffness:Float=1.0 )
		For Local m1:Int=0 To mIndex-1
			For Local m2:Int=m1 To mIndex-1
				ConAdd m1,m2,,stiffness
			Next
		Next
	End Method
	
	Rem
	bbdoc: Creates and adds a constraint to the TAdvBody
	returns: The index of the created constraint
	about: Creates a constraint associated with the TAdvBody and returns it's index.
	@mIndex1 and mIndex2 are the indices of the two masses to be constrained.
	See #TConstraint.Create for more information.
	End Rem
	Method ConAdd:Int( mIndex1:Int, mIndex2:Int, restLength:Float=-1, stiffness:Float=1.0 )
		If mArray[mIndex1] And mArray[mIndex2] And mIndex1<>mIndex2
			cIndex:+1
			Local c:TConstraint=TConstraint.Create( mArray[mIndex1], mArray[mIndex2], restLength, stiffness )
			If cArray.Length<cIndex Then cArray=cArray[..cIndex*2]
			cArray[cIndex-1]=c
			
			c.abID=aIndex
			c.index=cIndex-1
			
			Return cIndex-1
		Else
			Return -1
		EndIf
	End Method
	
	Rem
	bbdoc: Removes a constraint from the TAdvBody
	about: Removes the constraint at index from the TAdvBody.
	Remove will also remove any rigs that reference it.
	It should be noted that index values are not recycled.
	So removing a constraint will leave a gap in the index range.
	End Rem
	Method ConRemove( index:Int )
		If cArray[index]
			For Local rcount:Int=0 To rIndex-1
				If rArray[rcount]
					If rArray[rcount].con=cArray[index] Then RigRemove rcount
				EndIf
			Next
			ConFree index
			cArray[index].abID=-1
			cArray[index]=Null
		EndIf
	End Method
	
	Rem
	bbdoc: Get the maximum constraint index used by the TAdvBody
	returns: Maximum index
	about: note ConMaxIndex returns the maximum index that has been used by the TAdvBody.
	The actual maximum index may be lower.
	End Rem
	Method ConMaxIndex:Int()
		Return cIndex-1
	End Method
	
	Rem
	bbdoc: Swaps the index positions of two constraints.
	End Rem
	Method ConSwapIndex( index1:Int, index2:Int )
		If index1<0 Or index2<0 Or index1>=cIndex Or index2>=cIndex Then Return
		
		Local swap:TConstraint=cArray[index1]
		cArray[index1]=cArray[index2]
		cArray[index2]=swap
		
		If cArray[index1] Then cArray[index1].index=index1
		If cArray[index2] Then cArray[index2].index=index2
		
	End Method
	
	Rem
	bbdoc: Condenses the indices of the constraints.
	about: ConCleanIndex removes gaps in the index list.
	It should be noted that constraints index values will be changed by this method.
	End Rem
	Method ConCleanIndex()
		Local freeIndex:Int=0
		
		For Local count:Int=0 To cIndex-1
			
			If cArray[count]
				ConSwapIndex( freeIndex, count )
				
				freeIndex:+1
				
			EndIf
			
		Next
		
		cIndex=freeIndex
		
	End Method
	
	Rem
	bbdoc: Sees if constraint exists
	returns: True if constraint exists, false if otherwise.
	about: Not all indices within the index range are used by a TAdvBody.
	Use ConExists to see if @index is currently in use.
	End Rem
	Method ConExists:Int( index:Int )
		If index>=0 And index<cIndex And cArray[index] Then Return True
		Return False
	End Method
	
	Rem
	bbdoc: Gets a #TConstraint component.
	returns: The #TConstraint at index.
	about: ConGet returns the TConstraint at the given index.
	End Rem
	Method ConGet:TConstraint( index:Int )
		If cArray[index] Then Return cArray[index]
	End Method
	
	Method ConSetName( index:Int, n:String )
		If cArray[index] Then cArray[index].SetName n
	End Method
	
	Method ConGetName:String( index:Int )
		If cArray[index] Then Return cArray[index].GetName()
	End Method
	
	Method ConAssign( index:Int )
		If cArray[index] Then cArray[index].Assign Self
	End Method
	
	Method ConFree( index:Int )
		If cArray[index] Then cArray[index].Free Self
	End Method
	
	Method ConGetMass1:Int( index:Int )
		If cArray[index] Then Return cArray[index].GetMass1().index
	End Method
	
	Method ConGetMass2:Int( index:Int )
		If cArray[index] Then Return cArray[index].GetMass2().index
	End Method
	
	Method ConSetRestLength( index:Int, length:Float=-1 )
		If cArray[index] Then cArray[index].SetRestLength length
	End Method
	
	Method ConGetRestLength:Float( index:Int )
		If cArray[index] Then Return cArray[index].GetRestLength()
	End Method
	
	Method ConSetStiffness( index:Int, stiff:Float )
		If cArray[index] Then cArray[index].SetStiffness stiff
	End Method
	
	Method ConGetStiffness:Float( index:Int )
		If cArray[index] Then Return cArray[index].GetStiffness()
	End Method
	
	Method ConGetAngle:Float( index:Int )
		If cArray[index] Then Return cArray[index].GetAngle()
	End Method
	
	Method ConGetLength:Float( index:Int )
		If cArray[index] Then Return cArray[index].GetLength()
	End Method
	
	Method ConGetStress:Float( index:Int )
		If cArray[index] Then Return cArray[index].GetStress()
	End Method
	
	Method ConDraw( index:Int )
		If cArray[index] Then cArray[index].Draw
	End Method
	
	Method ConDrawAll()
		For Local c:TConstraint=EachIn cArray
			c.Draw
		Next
	End Method
	
	Rem
	bbdoc: Creates and adds a rig to the TAdvBody
	returns: The index of the created rig
	about: Creates a rig associated with the TAdvBody and returns it's index.
	@iIndex is the index of the image to use.
	@mIndex is the index of the mass that will act as the rigs handle.
	@cIndex is the index of the constraint the rig uses to determine it's orientation.
	@angle is the rotational offset offset of the rig.
	See #TRig.Create for more infomation. 
	End Rem
	Method RigAdd:Int( iIndex:Int, mIndex:Int, cIndex:Int=-1, angle:Float=0 )
		If iArray[iIndex] And mArray[mIndex]
			rIndex:+1
			If rArray.Length<rIndex Then rArray=rArray[..rIndex*2]
			
			Local con:TConstraint
			If cIndex>=0 Then con=cArray[cIndex]
			
			rArray[rIndex-1]=TRig.Create( iArray[iIndex], mArray[mIndex], con, angle )
			
			rArray[rIndex-1].abID=aIndex
			rArray[rIndex-1].index=rIndex-1
			
			Return rIndex-1
		Else
			Return -1
		EndIf
	End Method
	
	Rem
	bbdoc: Removes a rig from the TAdvBody
	about: Removes the rig at index from the TAdvBody.
	It should be noted that index values are not recycled.
	So removing a rig will leave a gap in the index range.
	End Rem
	Method RigRemove( index:Int )
		rArray[index].abID=-1
		rArray[index]=Null
	End Method
	
	Rem
	bbdoc: Get the maximum rig index used by the TAdvBody
	returns: Maximum index
	about: Note RigMaxIndex returns the maximum index that has been used by the TAdvBody.
	The actual maximum index may be lower.
	End Rem
	Method RigMaxIndex:Int()
		Return rIndex-1
	End Method
	
	Rem
	bbdoc: Swaps the index positions of two rigs.
	End Rem
	Method RigSwapIndex( index1:Int, index2:Int )
		If index1<0 Or index2<0 Or index1>=rIndex Or index2>=rIndex Then Return
		
		Local swap:TRig=rArray[index1]
		rArray[index1]=rArray[index2]
		rArray[index2]=swap
		
		If rArray[index1] Then rArray[index1].index=index1
		If rArray[index2] Then rArray[index2].index=index2
	End Method
	
	Rem
	bbdoc: Condenses the indices of the rigs.
	about: RigCleanIndex removes gaps in the index list.
	It should be noted that rigs index values will be changed by this method.
	End Rem
	Method RigCleanIndex()
		
		Local freeIndex:Int=0
		
		For Local count:Int=0 To rIndex-1
			
			If rArray[count]
				RigSwapIndex( freeIndex, count )
				
				freeIndex:+1
				
			EndIf
			
		Next
		
		rIndex=freeIndex
		
	End Method
	
	Rem
	bbdoc: Sees if rig exists
	returns: True if rig exists, false if otherwise.
	about: Not all indices within the index range are used by A TAdvBody.
	Use RigExists to see if @index is currently in use.
	End Rem
	Method RigExists:Int( index:Int )
		If index>=0 And index<rIndex And rArray[index] Then Return True
		Return False
	End Method
	
	Rem
	bbdoc: Gets a TRig component.
	returns: The TRig at index.
	about: RigGet returns the TRig at the given index.
	End Rem
	Method RigGet:TRig( index:Int )
		If rArray[index] Then Return rArray[index]
	End Method
	
	Method RigSetName( index:Int, n:String )
		If rArray[index] Then rArray[index].SetName n
	End Method
	
	Method RigGetName:String( index:Int )
		If rArray[index] Then Return rArray[index].GetName()
	End Method
	
	Method RigSetImage( index:Int, img:Int )
		If rArray[index] And iArray[img] Then rArray[index].SetImage iArray[img]
	End Method
	
	Method RigGetImage:Int( index:Int )
		If rArray[index]
			Local count:Int=0
			While iArray[count]<>rArray[index].image
				count:+1
			Wend
			Return count
		EndIf
	End Method
	
	Method RigSetMass( index:Int, m:Int )
		If rArray[index] And mArray[m] Then rArray[index].SetMass mArray[m]
	End Method
	
	Method RigGetMass:Int( index:Int )
		If rArray[index] Then Return rArray[index].GetMass().index
	End Method
	
	Method RigSetConstraint( index:Int, c:Int=-1 )
		If rArray[index]
			If c=-1
				rArray[index].SetConstraint
			Else
				rArray[index].SetConstraint cArray[c]
			EndIf
		EndIf
	End Method
	
	Method RigGetConstraint:Int( index:Int )
		If rArray[index]
			Local c:TConstraint=rArray[index].GetConstraint()
			If c
				Return c.index
			Else
				Return -1
			EndIf
		EndIf
	End Method
	
	Method RigSetAngle( index:Int, a:Float )
		If rArray[index] Then rArray[index].SetAngle a
	End Method
	
	Method RigGetAngle:Float( index:Int )
		If rArray[index] Then Return rArray[index].GetAngle()
	End Method
	
	Method RigSetScale( index:Int, sclX:Float, sclY:Float )
		If rArray[index] Then rArray[index].SetScale sclX,sclY
	End Method
	
	Method RigGetScale( index:Int, sclX:Float Var, sclY:Float Var )
		If rArray[index] Then rArray[index].GetScale sclX,sclY
	End Method
	
	Method RigSetColor( index:Int, red:Int, green:Int, blue:Int )
		If rArray[index] Then rArray[index].SetColor red,green,blue
	End Method
	
	Method RigGetColor( index:Int, red:Int Var, green:Int Var, blue:Int Var)
		If rArray[index] Then rArray[index].GetColor red,green,blue
	End Method
	
	Method RigSetAlpha( index:Int, alpha:Float )
		If rArray[index] Then rArray[index].SetAlpha alpha
	End Method
	
	Method RigGetAlpha:Float( index:Int )
		If rArray[index] Then Return rArray[index].GetAlpha()
	End Method
	
	Method RigSetBlend( index:Int, blend:Int )
		If rArray[index] Then rArray[index].SetBlend blend
	End Method
	
	Method RigGetBlend:Int( index:Int )
		If rArray[index] Then Return rArray[index].GetBlend()
	End Method
	
	Method RigDraw( index:Int )
		If rArray[index] Then rArray[index].Draw
	End Method
	
	Rem
	bbdoc: Adds an image to the TAdvBody
	returns: The index of the image
	about: An image must be associated with the TAdvBody before it can be used.
	The returned index can be used by a rig that is also associated with the body.
	@url can be an existing #TImage or the file name of an image.
	End Rem
	Method ImageAdd:Int( url:Object )
		Local img:TImage=TImage(url)
		If Not img Then img=LoadImage(url)
		If img
			iIndex:+1
			If iArray.Length<iIndex Then iArray=iArray[..iIndex*2]
			
			iArray[iIndex-1]=img
						
			Return iIndex-1

		Else
			Return -1
		EndIf
	End Method
	
	Rem
	bbdoc: Removes an image from the TAdvBody
	about: Removes the image at index from the TAdvBody.
	Remove will also remove any rigs that reference it.
	It should be noted that index values are not recycled.
	So removing an image will leave a gap in the index range.
	End Rem
	Method ImageRemove( index:Int )
		For Local rcount:Int=0 To rIndex-1
			If rArray[rcount]
				If rArray[rcount].image=iArray[index] Then RigRemove rcount
			EndIf
		Next
		iArray[index]=Null
	End Method
	
	Rem
	bbdoc: Get the maximum image index used by the TAdvBody
	returns: Maximum index
	about: Note ImageMaxIndex returns the maximum index that has been used by the TAdvBody.
	The actual maximum index may be lower.
	End Rem
	Method ImageMaxIndex:Int()
		Return iIndex-1
	End Method
	
	Rem
	bbdoc: Swaps the index positions of two rigs.
	End Rem
	Method ImageSwapIndex( index1:Int, index2:Int )
		If index1<0 Or index2<0 Or index1>=iIndex Or index2>=iIndex Then Return
		
		Local swap:TImage=iArray[index1]
		iArray[index1]=iArray[index2]
		iArray[index2]=swap
	End Method
	
	Rem
	bbdoc: Replaces one image with another.
	returns: True if replace was successful.
	about: Replaces the image at index with another image.
	This function is mainly useful when creating editors.
	End Rem
	Method ImageReplace:Int( index:Int, url:Object )
		Local img:TImage=TImage(url)
		If Not img Then img=LoadImage(url)
		If img
			If iArray[index]
				For Local rcount:Int=0 To rIndex-1
					If rArray[rcount]
						If rArray[rcount].image=iArray[index] Then rArray[rcount].image=img
					EndIf
				Next
			EndIf
			
			iArray[index]=img
			
			Return True

		Else
			Return False
		EndIf
	End Method
	
	Rem
	bbdoc: Condenses the indices of the images.
	about: ImageCleanIndex removes gaps in the index list.
	It should be noted that images index values will be changed by this method.
	End Rem
	Method ImageCleanIndex()
		Local freeIndex:Int=0
		
		For Local count:Int=0 To iIndex-1
			
			If iArray[count]
				ImageSwapIndex( freeIndex, count )
				
				freeIndex:+1
				
			EndIf
			
		Next
		
		iIndex=freeIndex
	End Method
	
	Rem
	bbdoc: Sees if image exists
	returns: True if image exists, false if otherwise.
	about: Not all indices within the index range are used by a TAdvBody.
	Use ImageExists to see if @index is currently in use.
	End Rem
	Method ImageExists:Int( index:Int )
		If index>=0 And index<iIndex And iArray[index] Then Return True
		Return False
	End Method
	
	Rem
	bbdoc: Gets the image at index
	returns: TImage at index
	about: Use ImageGet to retrieve the actual TImage of the image at index.
	This can then be used to perform image manipulation beyond the scope of PhysLite.
	End Rem
	Method ImageGet:TImage( index:Int )
		Return iArray[index]
	End Method
	
	Method New()
		If sNext>0
			sNext:-1
			aIndex=stack[sNext]
		Else
			aIndex=aNext
			aNext:+1
			If aNext=array.length
				array=array[..aNext*2]
				stack=stack[..aNext*2]
			EndIf
		EndIf
	End Method
	
End Type

