'Line Slider by David 'OddBall' Williamson
''''' '''''' '' ''''' ''''''''' ''''''''''
'Powered by PhysLite
'''''''' '' ''''''''
'The Examples are more to show the kinds of things that can be
'acheived using PhysLite than to teach specifics of the module.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

'Always SuperStrict
SuperStrict

'Framework
Framework BRL.GLMax2D	'DirectX sucks at drawing lines
Import BRL.PNGLoader
Import BRL.EventQueue
Import BRL.Timer

Import MaxGUI.Drivers
Import ODD.PhysLite

SetGraphicsDriver GLMax2DDriver()

'Include media into binary file
Incbin "media/sled.plb"
'Toolbar icons (c) Mark James - http://www.famfamfam.com/lab/icons/silk/
Incbin "media/toolbar.png"
Incbin "media/canvas.png"
Incbin "media/canvassmall.png"
Incbin "media/physlitelogo.png"

'Constants
Const TOOLBAR_NEW:Int=0
Const TOOLBAR_OPEN:Int=1
Const TOOLBAR_SAVE:Int=2
Const TOOLBAR_SAVEAS:Int=3
Const TOOLBAR_UNDO:Int=5
Const TOOLBAR_COLOR:Int=6
Const TOOLBAR_START:Int=8
Const TOOLBAR_PLAY:Int=9
Const TOOLBAR_PAUSE:Int=10

'Globals
Global window:TGadget
Global toolbar:TGadget
Global canvas:TGadget
Global timer:TTimer
Global red:Int
Global green:Int
Global blue:Int
Global bg:TImage
Global bgSmall:TImage
Global lines:TList
Global cLine:TZone
Global ox:Float
Global oy:Float
Global drawing:Int
Global scrolling:Int
Global playing:Int
Global cx:Float
Global cy:Float
Global sx:Float
Global sy:Float
Global tCount:Int
Global sled:TSled
Global logo:TImage
Global curFile:String
Global uCount:Int

'Start app.
GameInit

'Main loop
Repeat
	'Wait for events
	WaitEvent
	
	'Manually check timer. Timer events are bad!!
	Local t:Int=timer.Ticks()-tCount
	If t
		'Update game/physics
		GameUpdate
		PhysicsUpdate
		
		'Draw game
		DrawCanvas
		tCount=timer.Ticks()
	EndIf
Forever

Type TSled
	Field temp:TAdvBody=AdvBodyLoad("incbin::media/sled.plb")
	Field ab:TAdvBody
	
	'Create sled
	Function Create:TSled()
		Local s:TSled=New TSled
		
		s.ab=s.temp.Copy()
		s.ab.Activate
		s.ab.Position 0,0
		
		Return s
	End Function
	
	'Reset sled
	Method reset()
		ab.DeActivate
		ab.Free
		ab=temp.Copy()
		ab.Activate
		ab.Position 0,0
		cx=0
		cy=0
	End Method
	
	'Get X position of sled
	Method GetX:Float()
		Return ab.GetX()
	End Method
	
	'Get Y position of sled
	Method GetY:Float()
		Return ab.GetY()
	End Method
	
	'Draw sled
	Method Draw()
		ab.Draw
	End Method
End Type

'Game setup
Function GameInit()
	'Set apptitle and filenames
	AppTitle="Line Slider by David Williamson - "
	uCount=1
	curFile="Untitled_"+uCount
	uCount:+1
	
	'Create gadgets
	window=CreateWindow(AppTitle+curFile,0,0,800,600,Null,WINDOW_TITLEBAR|WINDOW_RESIZABLE|WINDOW_CENTER)
	toolbar=CreateToolBar("incbin::media/toolbar.png",0,0,264,24,window)
	SetToolBarTips toolbar,["New","Open","Save","Save As","","Undo","Select color","","Return to start","Play","Pause"]
	DisableGadgetItem toolbar,TOOLBAR_START
	DisableGadgetItem toolbar,TOOLBAR_PAUSE
	Local tbHeight:Int=GadgetHeight(toolbar)
	canvas=CreateCanvas(0,tbHeight,ClientWidth(window),ClientHeight(window)-tbHeight,window)
	SetGadgetLayout canvas,EDGE_ALIGNED,EDGE_ALIGNED,EDGE_ALIGNED,EDGE_ALIGNED
	
	'Set clear screen color
	SetGraphics CanvasGraphics(canvas)
	SetClsColor 255,255,255
	
	'Our timer
	timer=CreateTimer(60)
	StopTimer timer
	
	'Load background images
	bg=LoadImage("incbin::media/canvas.png",0)
	bgSmall=LoadImage("incbin::media/canvassmall.png",0)
	
	'Hook gadget and physics events
	AddHook EmitEventHook,DoGadgetEvents
	AddHook TColEvent.Hook,DoPhysicsEvents
	
	'Start PhysLite
	PhysicsBegin 800,600,.1,,2
	
	'Create list to store our lines
	lines=CreateList()
	
	'Create our hero
	sled=TSled.Create()
	
	'Advertise
	logo=LoadImage("incbin::media/physlitelogo.png",FILTEREDIMAGE|MIPMAPPEDIMAGE)
	
	'Activate window
	ActivateGadget window
	MaximizeWindow window
End Function

'Some pre-PhysicsUpdate stuff
Function GameUpdate()
	cx=sled.GetX()
	cy=sled.GetY()
	PhysicsSetBounds cx-10000,cy-10000,cx+10000,cy+10000
End Function

'Gadget events
Function DoGadgetEvents:Object( id:Int, data:Object, context:Object )
	Local event:TEvent=TEvent(data)
	Select event.ID
	Case EVENT_WINDOWSIZE
		'Draw canvas during resize
		RedrawGadget canvas
	Case EVENT_WINDOWCLOSE
		'Bye bye
		PhysicsEnd
		EndGraphics
		End
	Case EVENT_GADGETACTION
		'Toolbar actions
		If event.Source=toolbar
			Select event.Data
			'New canvas
			Case TOOLBAR_NEW
				'This should be wrapped into a function
				For Local l:TZone=EachIn lines
					l.Free
				Next
				lines.Clear
				cline=Null
				sled.Reset
				If timer Then StopTimer timer
				EnableGadgetItem toolbar,TOOLBAR_UNDO
				EnableGadgetItem toolbar,TOOLBAR_COLOR
				DisableGadgetItem toolbar,TOOLBAR_START
				EnableGadgetItem toolbar,TOOLBAR_PLAY
				DisableGadgetItem toolbar,TOOLBAR_PAUSE
				playing=False
				DrawCanvas
				curFile="Untitled_"+uCount
				uCount:+1
				SetGadgetText window,AppTitle+curFile
			'Open canvas
			Case TOOLBAR_OPEN
				'Another function wrap
				Local url:String=RequestFile("Select canvas to open","Canvas:lsc",False,AppDir+"/levels/")
				If FileType(url)=1
					Local file:TStream=ReadStream(url)
					Local header:String=file.ReadString(6)
					If header="CANVAS"
						For Local l:TZone=EachIn lines
							l.Free
						Next
						lines.Clear
						Local noz:Int=file.ReadInt()
						For Local count:Int=0 Until noz
							lines.AddLast ZoneLoad(file)
						Next
						cline=Null
						sled.Reset
						If timer Then StopTimer timer
						EnableGadgetItem toolbar,TOOLBAR_UNDO
						EnableGadgetItem toolbar,TOOLBAR_COLOR
						DisableGadgetItem toolbar,TOOLBAR_START
						EnableGadgetItem toolbar,TOOLBAR_PLAY
						DisableGadgetItem toolbar,TOOLBAR_PAUSE
						playing=False
						curFile=StripAll(url)
						SetGadgetText window,AppTitle+curFile
						DrawCanvas
					EndIf
					CloseStream file
				EndIf
			'Save canvas
			Case TOOLBAR_SAVE
				'No really, you should be wrapping these
				Local url:String=AppDir+"/levels/"+curFile+".lsc"
				Local file:TStream=WriteStream(url)
				file.WriteString "CANVAS"
				file.WriteInt lines.Count()
				For Local z:TZone=EachIn lines
					z.Save file,False
				Next
				CloseStream file
				curFile=StripAll(url)
				SetGadgetText window,AppTitle+curFile
			'Save canvas as
			Case TOOLBAR_SAVEAS
				'Oh well, have it your own way
				Local url:String=RequestFile("Save canvas as?","Canvas:lsc",True,AppDir+"/levels/"+curFile+".lsc")
				If url
					Local file:TStream=WriteStream(url)
					file.WriteString "CANVAS"
					file.WriteInt lines.Count()
					For Local z:TZone=EachIn lines
						z.Save file,False
					Next
					CloseStream file
					curFile=StripAll(url)
					SetGadgetText window,AppTitle+curFile
				EndIf
			'Undo last draw
			Case TOOLBAR_UNDO
				If Not lines.IsEmpty()
					Local l:TZone=TZone(lines.Last())
					If l.VertCount()<3
						l.Free
						lines.RemoveLast
					Else
						l.VertRemoveLast
					EndIf
					DrawCanvas
				EndIf
			'Pick draw color
			Case TOOLBAR_COLOR
				If RequestColor(red,green,blue)
					red=RequestedRed()
					green=RequestedGreen()
					blue=RequestedBlue()
				EndIf
			'Rewind playback to the start
			Case TOOLBAR_START
				sled.Reset
				StopTimer timer
				EnableGadgetItem toolbar,TOOLBAR_UNDO
				EnableGadgetItem toolbar,TOOLBAR_COLOR
				DisableGadgetItem toolbar,TOOLBAR_START
				EnableGadgetItem toolbar,TOOLBAR_PLAY
				DisableGadgetItem toolbar,TOOLBAR_PAUSE
				playing=False
				DrawCanvas
			'Start playback
			Case TOOLBAR_PLAY
				timer=CreateTimer(60)
				DisableGadgetItem toolbar,TOOLBAR_UNDO
				DisableGadgetItem toolbar,TOOLBAR_COLOR
				EnableGadgetItem toolbar,TOOLBAR_START
				DisableGadgetItem toolbar,TOOLBAR_PLAY
				EnableGadgetItem toolbar,TOOLBAR_PAUSE
				playing=True
			'Pause playback
			Case TOOLBAR_PAUSE
				StopTimer timer
				EnableGadgetItem toolbar,TOOLBAR_UNDO
				EnableGadgetItem toolbar,TOOLBAR_COLOR
				EnableGadgetItem toolbar,TOOLBAR_PLAY
				DisableGadgetItem toolbar,TOOLBAR_PAUSE
				playing=False
				DrawCanvas
			End Select
		EndIf
	Case EVENT_MOUSEDOWN
		Local mx:Int=event.x
		Local my:Int=event.y
		Local px:Float=PhysicsSimX(mx,my)
		Local py:Float=PhysicsSimY(mx,my)
		ox=px
		oy=py
		Select event.Data
		'Draw
		Case MOUSE_LEFT
			If Not playing
				drawing=True
				cLine=ZoneCreate([px,py,px,py],,0,2)
				cLine.SetColor red,green,blue
				cLine.SetBlend ALPHABLEND
				lines.AddLast cLine
				DrawCanvas
			EndIf
		'Scroll
		Case MOUSE_RIGHT
			If Not playing
				scrolling=True
				sx=px
				sy=py
			EndIf
		End Select
	Case EVENT_MOUSEMOVE
		Local mx:Int=event.x
		Local my:Int=event.y
		Local px:Float=PhysicsSimX(mx,my)
		Local py:Float=PhysicsSimY(mx,my)
		'Draw
		If drawing
			Local lIndex:Int=cLine.VertCount()-1
			cLine.VertPosition lIndex,px,py
			If ((px-ox)*(px-ox)+(py-oy)*(py-oy))>(10*10)
				cLine.VertAddLast px,py
				ox=px;oy=py
				DrawCanvas
			EndIf
		EndIf
		'Scroll
		If scrolling
			cx:-(px-sx)
			cy:-(py-sy)
			DrawCanvas
		EndIf
	Case EVENT_MOUSEUP
		'Stop drawing/scrolling
		Select event.Data
		Case MOUSE_LEFT
			drawing=False
			DrawCanvas
		Case MOUSE_RIGHT
			scrolling=False
		End Select
	Case EVENT_MOUSEENTER
		'Set mouse pointer
		If event.Source=canvas
			SetPointer POINTER_CROSS
		EndIf
	Case EVENT_MOUSELEAVE
		'Reset mouse pointer
		If event.Source=canvas
			SetPointer POINTER_DEFAULT
		EndIf
	Case EVENT_GADGETPAINT
		'Render the canvas
		DrawCanvas
	End Select
	Return data
End Function

'Physics events
Function DoPhysicsEvents:Object( id:Int, data:Object, context:Object )
	Local colEvent:TColEvent=TColEvent(data)
	Select colEvent.GetID()
	Case TColEvent.MASS_BOUNDS
	Case TColEvent.MASS_ZONE
		'If the man bumps his head he falls out
		If colEvent.GetMass1().GetName()="head"
			For Local count:Int=16 To sled.ab.ConMaxIndex()
				If sled.ab.ConExists(count)
					sled.ab.ConRemove(count)
				EndIf
			Next
		EndIf
	End Select
	
	Return data
End Function

'Render canvas
Function DrawCanvas()
	'Set up graphics
	SetGraphics CanvasGraphics(canvas)
	SetViewport 0,0,GraphicsWidth(),GraphicsHeight()
	SetBlend ALPHABLEND
	Local back:TImage
	Local zoom:Float
	'Zoom in for playback
	If playing
		back=bg
		zoom=.8
	'zoom out for drawing
	Else
		back=bgSmall
		zoom=.4
	EndIf
	'No need to PhysicsUpdateTransform, these functions wil do that too.
	PhysicsSetDrawZoom zoom
	PhysicsSetDrawCoords cx,cy
	
	'Clear screen
	Cls
	
	'Draw lines
	SetColor 255,255,255
	SetLineWidth 4*zoom
	For Local l:TZone=EachIn lines
		l.Draw
	Next
	
	'Draw sled
	sled.Draw
	
	'Apply canvas texture
	SetBlend shadeblend
	TileImage back,PhysicsScreenX(0,0),PhysicsScreenY(0,0)
	SetBlend alphablend
	
	'Advertise
	SetColor 255,255,255
	SetScale .5,.5
	DrawImage logo,GraphicsWidth()-90,GraphicsHeight()-70
	
	Flip
End Function