'Tutorial06: Lets scroll!
'''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'18th May 2007 - Revised 4th Feb 2009
'''''''''''''''''''''''''''''''''''''
'Description
''''''''''''
'An example of scrolling a simulation using PhysicsSetDrawCoords.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.PNGLoader
'Import "PhysLite/PhysLite.bmx"
Import ODD.PhysLite

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

PhysicsBegin 800,600,0.1,,2

Global mlist:TList=New TList
Global clist:TList=New TList
Global blist:TList=New TList

Global image:TImage=LoadImage("media/bmax.png",FILTEREDIMAGE)
SetImageHandle image,20,20

Type TBox Extends TBody
	Field rig:TRig
	
	Function Create:TBox( x:Float,y:Float )
		Local box:TBox=New TBox
		
		Local mass:TMass[4]
		mass[0]=MassCreate(x-10,y-10,10,20,1)
		mass[1]=MassCreate(x+10,y-10,10,20,1,mass[0].GetColGroup())
		mass[2]=MassCreate(x-10,y+10,10,20,1,mass[0].GetColGroup())
		mass[3]=MassCreate(x+10,y+10,10,20,1,mass[0].GetColGroup())
		
		Local con:TConstraint[6]
		con[0]=ConCreate(mass[0],mass[1])
		con[1]=ConCreate(mass[0],mass[2])
		con[2]=ConCreate(mass[0],mass[3])
		con[3]=ConCreate(mass[1],mass[2])
		con[4]=ConCreate(mass[1],mass[3])
		con[5]=ConCreate(mass[2],mass[3])
		For Local count:Int=0 To 5
			con[count].Assign box
		Next
		
		box.rig=RigCreate(image,mass[0],con[0])
		
		Return box
	End Function
	
End Type

For Local y:Int=100 To 340 Step 80
	For Local x:Int=80 To 720 Step 80
		blist.AddLast TBox.Create(x,y)
	Next
Next

Global start:TMass
Global finish:TMass
Global bridge:TBody
CreateBridge()

'Let's hide the mouse for this one
HideMouse

'This time all the magic happens in the main loop

Repeat
	
	PhysicsUpdate
	
	'We get the location where to draw from the mouse coords. We
	'could have just as easily used the location of an active
	'TMass object by using the GetPositionX and GetPositionY
	'methods instead of MouseX and MouseY.
	Local mx:Int=MouseX()
	Local my:Int=MouseY()
	
	'Using PhysicsSetDrawCoords to offset all the drawing commands
	'we create a scrolling effect without affecting the simulation.
	PhysicsSetDrawCoords mx,my
	
	'Now lets clear the screen.
	SetClsColor 0,0,0
	Cls
	
	'We now make ViewPort at the correct location and clear it in
	'grey. This will allow us to clearly see the simulation area.
	SetViewport 400-mx,300-my,800,600
	SetClsColor 127,127,127
	Cls
	
	'And it's that simple.
	
	SetColor 255,255,255
	For Local b:TBox=EachIn blist
		b.rig.Draw
	Next
	
	DrawTheRest
	
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end

Function CreateBridge()
	bridge=New TBody
	start=MassCreate(240,400,10,20,1,99)
	finish=MassCreate(560,400,10,20,1,99)
	Local m1:TMass=start
	Local m2:TMass
	Local c:TConstraint
	For Local x:Int=240 To 560 Step 40
		m2=MassCreate(x,440,10,20,1,99)
		c=ConCreate(m1,m2)
		c.Assign bridge
		mlist.AddLast m2
		clist.AddLast c
		m1=m2
	Next
	c=ConCreate(m1,finish)
	c.Assign bridge
	clist.AddLast c
	start.Freeze
	finish.Freeze
End Function

Function DrawTheRest()
	SetColor 0,0,255
	For Local m:TMass=EachIn mlist
		m.Draw
	Next
	SetColor 255,0,0
	start.Draw
	finish.Draw
	SetColor 255,255,255
	For Local c:TConstraint=EachIn clist
		c.Draw
	Next
End Function
