'Tutorial08: We want information
''''''''''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'19th May 2007 - Revised 4th Feb 2009
'''''''''''''''''''''''''''''''''''''
'Description
''''''''''''
'This tutorial covers retrieving the information you need from objects.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.PNGLoader
'Import "PhysLite/PhysLite.bmx"
Import ODD.PhysLite

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

PhysicsBegin 800,600,0.1,,2

Global mlist:TList=New TList
Global clist:TList=New TList
Global blist:TList=New TList

Global image:TImage=LoadImage("media/bmax.png",FILTEREDIMAGE)
SetImageHandle image,20,20

Type TBox Extends TBody
	Field rig:TRig
	
	'We need to get our info from somewhere so we set aside a place
	'for a TMass and TConstraint objects.
	Field mass:TMass
	Field con:TConstraint
	
	Function Create:TBox( x:Float,y:Float )
		Local box:TBox=New TBox
		
		Local mass:TMass[4]
		mass[0]=MassCreate(x-10,y-10,10,20,1)
		mass[1]=MassCreate(x+10,y-10,10,20,1,mass[0].GetColGroup())
		mass[2]=MassCreate(x-10,y+10,10,20,1,mass[0].GetColGroup())
		mass[3]=MassCreate(x+10,y+10,10,20,1,mass[0].GetColGroup())
		
		Local con:TConstraint[6]
		con[0]=ConCreate(mass[0],mass[1])
		con[1]=ConCreate(mass[0],mass[2])
		con[2]=ConCreate(mass[0],mass[3])
		con[3]=ConCreate(mass[1],mass[2])
		con[4]=ConCreate(mass[1],mass[3])
		con[5]=ConCreate(mass[2],mass[3])
		For Local count:Int=0 To 5
			con[count].Assign box
		Next
		
		box.rig=RigCreate(image,mass[0],con[0])
		
		'Now I choose a TMass and a TConstraint that we will use to
		'retrieve info on the TBox. I have chosen con[0] as it will
		'have an angle of 0 when the box is the right way up.
		box.mass=mass[0]
		box.con=con[0]
		
		Return box
	End Function
	
End Type

For Local y:Int=100 To 340 Step 80
	For Local x:Int=80 To 720 Step 80
		blist.AddLast TBox.Create(x,y)
	Next
Next

HideMouse
Global arrow:TImage=LoadImage("media/arrow.png",FILTEREDIMAGE)
SetImageHandle arrow,64,64

Repeat
	
	Local mx:Int=MouseX()-400
	Local my:Int=MouseY()-300
	
	PhysicsSetGravity mx/6000.0,my/6000.0
	
	PhysicsUpdate
	
	SetClsColor 127,127,127
	Cls
	
	For Local b:TBox=EachIn blist
		
		'Instead of just printing the info we get lets have a bit of fun.
		'Let's color each box based on it's velocity, position, and angle.
		Local red:Int,green:Int,blue:Int
		
		'First up is velocity. We could just get the x or y component of
		'a TMass objects velocity using GetVelocityX/Y, but for our purposes
		'we'll get the overall velocity. Now if velocity is over 4 units per
		'time step then the red value of the box is set to maximum. The units
		'used by the simulation could be arbitrary but in this tutorial 1
		'unit equals 1 pixel.
		If b.mass.GetVelocity()>4 Then red=255
		
		'Next is position. Using GetPositionY we test to see if the TMass objects
		'position is greater than 400. As we are mapping the simulation exactly to
		'screen coordinates this means that positive Y is actually pointing down and
		'therefore if our TMass is below half way the green value is set to maximum.
		If b.mass.GetPositionY()>400 Then green=255
		
		'Finally we test the angle. Now even on a flat surface it is unlikely that
		'our box will be exactly 0 so we'll give it a little leeway. As TMass objects
		'don't have a direction we need to use a TConstraint for this. If the 
		'TConstraint is close to 0 angle then we set the blue value to maximum.
		If b.con.GetAngle()>-2 And b.con.GetAngle()<2 Then blue=255
		
		'Using these values we set the color of the object. If none the criteria were
		'met the box will be black. if all the criteria are met the object will be
		'white. Usually only some of the criteria will be met cause the usual color
		'blends
		b.rig.SetColor red,green,blue
		
		'There is other info we could have used too, like constraint stress or length,
		'but as we are using full stiffness in most case these will be constant.
		
		b.rig.Draw
	Next
	
	SetRotation 0
	SetColor 255,255,255
	SetScale Sqr(mx*mx+my*my)/400,1
	SetRotation ATan2(my,mx)
	DrawImage arrow,MouseX()/2+200,MouseY()/2+150
	SetScale 1,1
	
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end
