'Tutorial09: Control. You must learn control
''''''''''''''''''''''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'27th May 2007 - Revised 4th Feb 2009
'''''''''''''''''''''''''''''''''''''
'Description
''''''''''''
'Because games aren't fun unless you are in control of the action.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.PNGLoader
'Import "PhysLite/PhysLite.bmx"
Import ODD.PhysLite

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

'Let's have no gravity this time
PhysicsBegin 800,600,0,0,2

'After you've tried this tutorial with no gravity try
'uncommenting the line below to add a little.
'PhysicsSetGravity 0,0.02

'We need a list to keep the scenery in.
Global zlist:TList=New TList

Global image:TImage=LoadImage("media/bmax.png",FILTEREDIMAGE)
SetImageHandle image,20,20

Type TBox Extends TBody
	Field rig:TRig
	
	'We need two masses and one constraint for this tutorial.
	Field lmass:TMass
	Field rmass:TMass
	Field con:TConstraint
	
	Function Create:TBox( x:Float,y:Float )
		Local box:TBox=New TBox
		
		Local mass:TMass[4]
		mass[0]=MassCreate(x-10,y-10,10,20,1)
		mass[1]=MassCreate(x+10,y-10,10,20,1,mass[0].GetColGroup())
		mass[2]=MassCreate(x-10,y+10,10,20,1,mass[0].GetColGroup())
		mass[3]=MassCreate(x+10,y+10,10,20,1,mass[0].GetColGroup())
		
		Local con:TConstraint[6]
		con[0]=ConCreate(mass[0],mass[1])
		con[1]=ConCreate(mass[2],mass[0])
		con[2]=ConCreate(mass[0],mass[3])
		con[3]=ConCreate(mass[1],mass[2])
		con[4]=ConCreate(mass[3],mass[1])
		con[5]=ConCreate(mass[2],mass[3])
		For Local count:Int=0 To 5
			con[count].Assign box
		Next
		
		box.rig=RigCreate(image,mass[0],con[0])
		
		'Now we put the bottom left and right masses in the correct field.
		box.lmass=mass[2]
		box.rmass=mass[3]
		'And we do the same with the constraint.
		'This time we've chosen a constraint that points up.
		box.con=con[1]
		
		Return box
	End Function
	
End Type

'We only need one box this time.
Global box:TBox=TBox.Create(400,300)

'And lets make some scenery to move around and bump into.
zlist.AddLast ZoneCreate([100.0,100.0,300.0,100.0,300.0,150.0,200.0,150.0,200.0,250.0,100.0,250.0],TZone.FILL)
zlist.AddLast ZoneCreate([700.0,100.0,500.0,100.0,500.0,150.0,600.0,150.0,600.0,250.0,700.0,250.0],TZone.FILL)
zlist.AddLast ZoneCreate([100.0,500.0,300.0,500.0,300.0,450.0,200.0,450.0,200.0,350.0,100.0,350.0],TZone.FILL)
zlist.AddLast ZoneCreate([700.0,500.0,500.0,500.0,500.0,450.0,600.0,450.0,600.0,350.0,700.0,350.0],TZone.FILL)

Repeat
	
	'First we get the direction the box is facing.
	Local angle:Float=box.con.GetAngle()
	'And lets use that to make a normal vector.
	Local x:Float=Cos(angle)
	Local y:Float=Sin(angle)
	
	'Ok now time for the magic.
	'If the player is pressing the up cursor key
	If KeyDown(KEY_UP)
		'Then move both masses equally
		box.lmass.Move x*0.1,y*0.1
		box.rmass.Move x*0.1,y*0.1
		'The move method not only changes a masses position,
		'but also adjusts its velocity accordingly.
		'If we wanted to move the box without affecting its
		'velocity we would have used Translate.
		'box.lmass.Translate x,y
		'box.rmass.Translate x,y
		'We could have also used AddVelocity to accelerate without
		'moving the box instantly.
		'box.lmass.AddVelocity x*0.1,y*0.1
		'box.rmass.AddVelocity x*0.1,y*0.1
	EndIf
	
	'If the player is pressing left.
	If KeyDown(KEY_LEFT)
		'Then spin the box anti-clockwise
		box.lmass.Move -x*0.1,-y*0.1
		box.rmass.Move x*0.1,y*0.1
	EndIf
	
	'If the player is pressing right.
	If KeyDown(KEY_RIGHT)
		'Then spin the box clockwise
		box.lmass.Move x*0.1,y*0.1
		box.rmass.Move -x*0.1,-y*0.1
	EndIf
	
	PhysicsUpdate
	
	SetClsColor 127,127,127
	Cls
	
	'Lets draw our box
	SetColor 255,255,255
	box.rig.Draw
	
	SetRotation 0
	DrawTheRest
	
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end

Function DrawTheRest()
	SetColor 255,0,0
	For Local zone:TZone=EachIn zlist
		zone.Draw
	Next
End Function
