'Tutorial11: It's Morphing Time.
''''''''''''''''''''''''''''''''
'By David 'OddBall' Williamson
''''''''''''''''''''''''''''''
'25th June 2007 - Revised 4th Feb 2009
''''''''''''''''''''''''''''''''''''''
'Description
''''''''''''
'More control methods. This time using the mouse to move and deform a
'body.
'
'This tutorial assumes you have read all previous tutorials in the series
'and as such only parts where code has changed will be commented.
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

SuperStrict

Framework BRL.GLMax2D
Import BRL.Random
'Import "PhysLite/PhysLite.bmx"
Import ODD.PhysLite

?win32
SetGraphicsDriver GLMax2DDriver()
?
Graphics 800,600,,60

PhysicsBegin 800,600,0.1,,1

'Ok. Lets make a body to work with.
Global body:TBody=New TBody

'First we need some TMass objects.
Global massList:TList=New TList
For Local count:Int=0 To 5
	'I'm just creating them at random locations
	massList.AddLast MassCreate(Rnd(300,500),Rnd(200,400),10,10,1,1)
Next

'Now we link them up.
For Local m1:TMass=EachIn massList
	For Local m2:TMass=EachIn massList
		If m1<>m2 Then ConCreate(m1,m2,,0.5).Assign body
	Next
Next

Repeat
	
	Local mx:Int=MouseX()
	Local my:Int=MouseY()
	Local button:Int=0
	If MouseDown(MOUSE_LEFT) Then button=1
	If MouseDown(MOUSE_RIGHT) Then button=2
	
	Local nearDist:Float=400
	Local nearMass:TMass
	'If the left mouse button is pressed we are going to move the selected mass
	'without altering the shape of the body. This will move the whole body.
	If button=1
		'Now we need to find the closest mass that is within 20 pixels.
		For Local m:TMass=EachIn massList
			Local dist:Float=(m.GetX()-mx)*(m.GetX()-mx)+(m.GetY()-my)*(m.GetY()-my)
			If dist<nearDist
				nearDist=dist
				nearMass=m
			EndIf
		Next
		'If there is a mass within 20 pixels we position it at the mouse coords.
		If nearMass Then nearMass.Position mx,my
	'If the right mouse button is pressed we are again going to move the selected
	'mass, but this time we will deform the body accordingly.
	ElseIf button=2
		'Agan we find the closest mass that is within 20 pixels
		For Local m:TMass=EachIn massList
			Local dist:Float=(m.GetX()-mx)*(m.GetX()-mx)+(m.GetY()-my)*(m.GetY()-my)
			If dist<nearDist
				nearDist=dist
				nearMass=m
			EndIf
		Next
		If nearMass
			'Again we move the mass
			nearMass.Position mx,my
			'We can use the method GetConstraints to retrieve a list of all the
			'constraints in a TBody.
			For Local c:TConstraint=EachIn body.GetConstraints()
				'This time we will alter the rest length of the constraints attached
				'to the selected mass.
				If c.GetMass1()=nearMass Or c.GetMass2()=nearMass Then c.SetRestLength
			Next
		EndIf
	EndIf
	
	'Only update the simulation if the body isn't being deformed.
	If button<>2 Or Not nearMass Then PhysicsUpdate
	
	SetClsColor 127,127,127
	Cls
	
	'Lets draw our body.
	SetColor 0,0,255
	For Local m:TMass=EachIn massList
		m.Draw
	Next
	SetColor 255,0,0
	If nearMass Then nearMass.Draw
	SetColor 255,255,255
	For Local c:TConstraint=EachIn body.GetConstraints()
		If button=2 And (c.GetMass1()=nearMass Or c.GetMass2()=nearMass)
			SetColor 0,255,0
		Else
			SetColor 255,255,255
		EndIf
		c.Draw
	Next
	
	Flip
	
Until KeyHit(KEY_ESCAPE)

PhysicsEnd

EndGraphics

End
'The end
